// Copyright (C) 2003 Free Software Foundation, Inc.
//
// This file is part of the GNU ISO C++ Library.  This library is free
// software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the
// Free Software Foundation; either version 2, or (at your option)
// any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License along
// with this library; see the file COPYING.  If not, write to the Free
// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
// USA.

// As a special exception, you may use this file as part of a free software
// library without restriction.  Specifically, if other files instantiate
// templates or use macros or inline functions from this file, or you compile
// this file and link it with other files to produce an executable, this
// file does not by itself cause the resulting executable to be covered by
// the GNU General Public License.  This exception does not however
// invalidate any other reasons why the executable file might be covered by
// the GNU General Public License.

#include <istream>
#include <streambuf>
#include <sstream>
#include <testsuite_hooks.h>
#include <testsuite_io.h>

using namespace std;

void test14()
{
  bool test __attribute__((unused)) = true;
  __gnu_test::fail_streambuf bib;
  istream stream(&bib);
  stream.exceptions(ios_base::badbit);
  stringbuf sbuf("", ios_base::out);

  try
    {
      stream >> &sbuf;
    }
  catch (...)
    { VERIFY(false); }

  VERIFY(stream.rdstate() & ios_base::failbit);
  VERIFY((stream.rdstate() & ios_base::badbit) == 0);
}

void test16()
{
  bool test __attribute__((unused)) = true;
  istringstream stream("foo, bar, qux");
  stream.exceptions(ios_base::badbit);
  __gnu_test::fail_streambuf bob;

  try
    {
      stream >> &bob;
    }
  catch (...)
    { VERIFY(false); }

  VERIFY(stream.rdstate() & ios_base::failbit);
  VERIFY((stream.rdstate() & ios_base::badbit) == 0);
}

// libstdc++/9371
int main()
{
  test14();
  test16();
  return 0;
}
